@@ -1,10 +1,12 @@
 ;;; blist.el --- Display bookmarks in an ibuffer way  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2021  Free Software Foundation, Inc.
+;; Copyright (C) 2021, 2022, 2023  Free Software Foundation, Inc.
 
-;; Author: Durand <mmemmew@gmail.com>
+;; Author: Durand <durand@jsdurand.xyz>
 ;; Keywords: convenience
-;; Version: 0.2
+;; URL: https://gitlab.com/mmemmew/blist
+;; Package-Requires: ((ilist "0.1") (emacs "24"))
+;; Version: 0.3
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -121,6 +123,12 @@ Used by `ilist-dag' to define an automatic filter group."
   (save-match-data
     (let* ((handler (bookmark-get-handler element))
            (handler-name (and handler (format "%S" handler)))
+           (handler-type
+            (cond
+             ;; This is added in Emacs 29, so we test if this function
+             ;; is defined.
+             ((functionp #'bookmark-type-from-full-record)
+              (bookmark-type-from-full-record element))))
            (location (bookmark-location element)))
       ;; replace repeating parts
       (cond
@@ -136,6 +144,8 @@ Used by `ilist-dag' to define an automatic filter group."
              (string-match "\\.\\([^.]+\\)\\'" location))
         (setq handler-name (match-string 1 location))))
       (cond
+       ;; If the type is available, we use it as the group header.
+       (handler-type)
        (handler-name
         (cond
          ;; Some special cases
@@ -144,7 +154,21 @@ Used by `ilist-dag' to define an automatic filter group."
          ((<= (length handler-name) 3) (upcase handler-name))
          ((capitalize handler-name))))))))
 
-;; the function defined above
+;;;;; An alternative, type-only, automatic grouping
+
+;; An alternative automatic group that only uses types of handlers as
+;; group headers.
+
+;; NOTE: bookmark-type-from-full-record is added in Emacs 29, so it is
+;; unavailable before this version, and hence this automatic filter
+;; group would always use the default group.
+
+(ilist-dag "blist-type-only" "Default" #'blist-filter-sorter-default
+  (cond
+   ((functionp #'bookmark-type-from-full-record)
+    (bookmark-type-from-full-record element))))
+
+;; For discoverability
 (defalias 'blist-automatic-filter-groups-default
   #'ilist-automatic-group-blist-default)
 
@@ -647,6 +671,8 @@ used as a `revert-buffer-function'."
   (define-key map (vector 'backtab) #'blist-prev-group)
   (define-key map (vector 'return) #'blist-return)
   (define-key map (vector 'S-return) #'blist-toggle-other-groups)
+  (define-key map (vector ?\C-m) #'blist-return)
+  (define-key map (vector ?\C-\S-m) #'blist-toggle-other-groups)
   (define-key map (vector ?o) #'blist-open-other-window)
   (define-key map (vector ?v) #'blist-select)
   (define-key map (vector ?r) #'blist-rename)
@@ -1277,7 +1303,8 @@ progress."
                  bookmark-alist))
          (name-len (length names))
          (new-len (length new-list))
-         (table (make-hash-table :test 'equal :size name-len))
+         (table (make-hash-table
+                 :test 'equal :size (+ name-len new_len)))
          (count 0))
     ;; use a hash table so that testing for membership is a constant
     ;; time operation
@@ -1943,5 +1970,9 @@ stop at."
   (blist-assert-mode)
   (ilist-forward-line arg blist-movement-cycle nil))
 
+;;;; blist-show-info
+
+;; TODO: Display counts about each groups.
+
 (provide 'blist)
 ;;; blist.el ends here
